@@ -1,29 +1,20 @@
-// Copyright 2013-2018, The Gtk-rs Project Developers.
-// See the COPYRIGHT file at the top-level directory of this distribution.
-// Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
+// Take a look at the license at the top of the repository in the LICENSE file.
 
-use fragile::Fragile;
-use gio_sys;
-use glib;
+use crate::Cancellable;
+use crate::Socket;
+use crate::SocketAddress;
 use glib::object::{Cast, IsA};
 use glib::translate::*;
-use glib_sys;
 use std::cell::RefCell;
 use std::mem::transmute;
 #[cfg(all(not(unix), feature = "dox"))]
 use std::os::raw::c_int;
 #[cfg(all(not(windows), feature = "dox"))]
 use std::os::raw::c_void;
+use std::pin::Pin;
 use std::ptr;
-use Cancellable;
-use Error;
-use Socket;
-use SocketAddress;
 
-#[cfg(feature = "futures")]
-use futures::future::Future;
-#[cfg(feature = "futures")]
-use futures::stream::Stream;
+use futures_core::stream::Stream;
 
 #[cfg(unix)]
 use std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
@@ -33,10 +24,10 @@ use std::os::windows::io::{AsRawSocket, FromRawSocket, IntoRawSocket, RawSocket}
 
 impl Socket {
     #[cfg(any(unix, feature = "dox"))]
-    pub unsafe fn new_from_fd<T: IntoRawFd>(fd: T) -> Result<Socket, Error> {
+    pub unsafe fn from_fd<T: IntoRawFd>(fd: T) -> Result<Socket, glib::Error> {
         let fd = fd.into_raw_fd();
         let mut error = ptr::null_mut();
-        let ret = gio_sys::g_socket_new_from_fd(fd, &mut error);
+        let ret = ffi::g_socket_new_from_fd(fd, &mut error);
         if error.is_null() {
             Ok(from_glib_full(ret))
         } else {
@@ -44,10 +35,10 @@ impl Socket {
         }
     }
     #[cfg(any(windows, feature = "dox"))]
-    pub unsafe fn new_from_socket<T: IntoRawSocket>(socket: T) -> Result<Socket, Error> {
+    pub unsafe fn from_socket<T: IntoRawSocket>(socket: T) -> Result<Socket, glib::Error> {
         let socket = socket.into_raw_socket();
         let mut error = ptr::null_mut();
-        let ret = gio_sys::g_socket_new_from_fd(socket as i32, &mut error);
+        let ret = ffi::g_socket_new_from_fd(socket as i32, &mut error);
         if error.is_null() {
             Ok(from_glib_full(ret))
         } else {
@@ -59,104 +50,117 @@ impl Socket {
 #[cfg(any(unix, feature = "dox"))]
 impl AsRawFd for Socket {
     fn as_raw_fd(&self) -> RawFd {
-        unsafe { gio_sys::g_socket_get_fd(self.to_glib_none().0) as _ }
+        unsafe { ffi::g_socket_get_fd(self.to_glib_none().0) as _ }
     }
 }
 
 #[cfg(any(windows, feature = "dox"))]
 impl AsRawSocket for Socket {
     fn as_raw_socket(&self) -> RawSocket {
-        unsafe { gio_sys::g_socket_get_fd(self.to_glib_none().0) as _ }
+        unsafe { ffi::g_socket_get_fd(self.to_glib_none().0) as _ }
     }
 }
 
 pub trait SocketExtManual: Sized {
-    fn receive<B: AsMut<[u8]>>(
+    #[doc(alias = "g_socket_receive")]
+    fn receive<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error>;
-    fn receive_from<B: AsMut<[u8]>>(
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error>;
+    #[doc(alias = "g_socket_receive_from")]
+    fn receive_from<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<(usize, SocketAddress), Error>;
-    fn receive_with_blocking<B: AsMut<[u8]>>(
+        cancellable: Option<&C>,
+    ) -> Result<(usize, SocketAddress), glib::Error>;
+    #[doc(alias = "g_socket_receive_with_blocking")]
+    fn receive_with_blocking<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
         blocking: bool,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error>;
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error>;
 
-    fn send<B: AsRef<[u8]>>(
+    #[doc(alias = "g_socket_send")]
+    fn send<B: AsRef<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error>;
-    fn send_to<B: AsRef<[u8]>, P: IsA<SocketAddress>>(
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error>;
+    #[doc(alias = "g_socket_send_to")]
+    fn send_to<B: AsRef<[u8]>, P: IsA<SocketAddress>, C: IsA<Cancellable>>(
         &self,
         address: Option<&P>,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error>;
-    fn send_with_blocking<B: AsRef<[u8]>>(
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error>;
+    #[doc(alias = "g_socket_send_with_blocking")]
+    fn send_with_blocking<B: AsRef<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
         blocking: bool,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error>;
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error>;
 
     #[cfg(any(unix, feature = "dox"))]
-    fn get_fd<T: FromRawFd>(&self) -> T;
+    #[cfg_attr(feature = "dox", doc(cfg(unix)))]
+    #[doc(alias = "get_fd")]
+    #[doc(alias = "g_socket_get_fd")]
+    fn fd<T: FromRawFd>(&self) -> T;
 
     #[cfg(any(windows, feature = "dox"))]
-    fn get_socket<T: FromRawSocket>(&self) -> T;
+    #[cfg_attr(feature = "dox", doc(cfg(windows)))]
+    #[doc(alias = "get_socket")]
+    #[doc(alias = "g_socket_get_fd")]
+    fn socket<T: FromRawSocket>(&self) -> T;
 
-    fn create_source<F>(
+    #[doc(alias = "g_socket_create_source")]
+    fn create_source<F, C>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         name: Option<&str>,
         priority: glib::Priority,
         func: F,
     ) -> glib::Source
     where
-        F: FnMut(&Self, glib::IOCondition) -> glib::Continue + 'static;
+        F: FnMut(&Self, glib::IOCondition) -> glib::Continue + 'static,
+        C: IsA<Cancellable>;
 
-    #[cfg(feature = "futures")]
-    fn create_source_future(
+    fn create_source_future<C: IsA<Cancellable>>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         priority: glib::Priority,
-    ) -> Box<dyn Future<Output = glib::IOCondition> + std::marker::Unpin>;
+    ) -> Pin<Box<dyn std::future::Future<Output = glib::IOCondition> + 'static>>;
 
-    #[cfg(feature = "futures")]
-    fn create_source_stream(
+    fn create_source_stream<C: IsA<Cancellable>>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         priority: glib::Priority,
-    ) -> Box<dyn Stream<Item = glib::IOCondition> + std::marker::Unpin>;
+    ) -> Pin<Box<dyn Stream<Item = glib::IOCondition> + 'static>>;
 }
 
 impl<O: IsA<Socket>> SocketExtManual for O {
-    fn receive<B: AsMut<[u8]>>(
+    fn receive<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         mut buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let buffer = buffer.as_mut();
         let buffer_ptr = buffer.as_mut_ptr();
         let count = buffer.len();
         unsafe {
             let mut error = ptr::null_mut();
-            let ret = gio_sys::g_socket_receive(
+            let ret = ffi::g_socket_receive(
                 self.as_ref().to_glib_none().0,
                 buffer_ptr,
                 count,
-                cancellable.0,
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -167,12 +171,13 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }
     }
 
-    fn receive_from<B: AsMut<[u8]>>(
+    fn receive_from<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         mut buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<(usize, SocketAddress), Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<(usize, SocketAddress), glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let buffer = buffer.as_mut();
         let buffer_ptr = buffer.as_mut_ptr();
         let count = buffer.len();
@@ -180,12 +185,12 @@ impl<O: IsA<Socket>> SocketExtManual for O {
             let mut error = ptr::null_mut();
             let mut addr_ptr = ptr::null_mut();
 
-            let ret = gio_sys::g_socket_receive_from(
+            let ret = ffi::g_socket_receive_from(
                 self.as_ref().to_glib_none().0,
                 &mut addr_ptr,
                 buffer_ptr,
                 count,
-                cancellable.0,
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -196,24 +201,25 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }
     }
 
-    fn receive_with_blocking<B: AsMut<[u8]>>(
+    fn receive_with_blocking<B: AsMut<[u8]>, C: IsA<Cancellable>>(
         &self,
         mut buffer: B,
         blocking: bool,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let buffer = buffer.as_mut();
         let buffer_ptr = buffer.as_mut_ptr();
         let count = buffer.len();
         unsafe {
             let mut error = ptr::null_mut();
-            let ret = gio_sys::g_socket_receive_with_blocking(
+            let ret = ffi::g_socket_receive_with_blocking(
                 self.as_ref().to_glib_none().0,
                 buffer_ptr,
                 count,
-                blocking.to_glib(),
-                cancellable.0,
+                blocking.into_glib(),
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -224,23 +230,24 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }
     }
 
-    fn send<B: AsRef<[u8]>>(
+    fn send<B: AsRef<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let (count, buffer_ptr) = {
             let slice = buffer.as_ref();
             (slice.len(), slice.as_ptr())
         };
         unsafe {
             let mut error = ptr::null_mut();
-            let ret = gio_sys::g_socket_send(
+            let ret = ffi::g_socket_send(
                 self.as_ref().to_glib_none().0,
                 mut_override(buffer_ptr),
                 count,
-                cancellable.0,
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -251,13 +258,14 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }
     }
 
-    fn send_to<B: AsRef<[u8]>, P: IsA<SocketAddress>>(
+    fn send_to<B: AsRef<[u8]>, P: IsA<SocketAddress>, C: IsA<Cancellable>>(
         &self,
         address: Option<&P>,
         buffer: B,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let (count, buffer_ptr) = {
             let slice = buffer.as_ref();
             (slice.len(), slice.as_ptr())
@@ -265,12 +273,12 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         unsafe {
             let mut error = ptr::null_mut();
 
-            let ret = gio_sys::g_socket_send_to(
+            let ret = ffi::g_socket_send_to(
                 self.as_ref().to_glib_none().0,
                 address.map(|p| p.as_ref()).to_glib_none().0,
                 mut_override(buffer_ptr),
                 count,
-                cancellable.0,
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -281,25 +289,26 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }
     }
 
-    fn send_with_blocking<B: AsRef<[u8]>>(
+    fn send_with_blocking<B: AsRef<[u8]>, C: IsA<Cancellable>>(
         &self,
         buffer: B,
         blocking: bool,
-        cancellable: Option<&Cancellable>,
-    ) -> Result<usize, Error> {
-        let cancellable = cancellable.to_glib_none();
+        cancellable: Option<&C>,
+    ) -> Result<usize, glib::Error> {
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         let (count, buffer_ptr) = {
             let slice = buffer.as_ref();
             (slice.len(), slice.as_ptr())
         };
         unsafe {
             let mut error = ptr::null_mut();
-            let ret = gio_sys::g_socket_send_with_blocking(
+            let ret = ffi::g_socket_send_with_blocking(
                 self.as_ref().to_glib_none().0,
                 mut_override(buffer_ptr),
                 count,
-                blocking.to_glib(),
-                cancellable.0,
+                blocking.into_glib(),
+                gcancellable.0,
                 &mut error,
             );
             if error.is_null() {
@@ -311,89 +320,90 @@ impl<O: IsA<Socket>> SocketExtManual for O {
     }
 
     #[cfg(any(unix, feature = "dox"))]
-    fn get_fd<T: FromRawFd>(&self) -> T {
-        unsafe { FromRawFd::from_raw_fd(gio_sys::g_socket_get_fd(self.as_ref().to_glib_none().0)) }
+    #[cfg_attr(feature = "dox", doc(cfg(unix)))]
+    fn fd<T: FromRawFd>(&self) -> T {
+        unsafe { FromRawFd::from_raw_fd(ffi::g_socket_get_fd(self.as_ref().to_glib_none().0)) }
     }
 
     #[cfg(any(windows, feature = "dox"))]
-    fn get_socket<T: FromRawSocket>(&self) -> T {
+    #[cfg_attr(feature = "dox", doc(cfg(windows)))]
+    fn socket<T: FromRawSocket>(&self) -> T {
         unsafe {
-            FromRawSocket::from_raw_socket(
-                gio_sys::g_socket_get_fd(self.as_ref().to_glib_none().0) as _
-            )
+            FromRawSocket::from_raw_socket(ffi::g_socket_get_fd(self.as_ref().to_glib_none().0) as _)
         }
     }
 
-    fn create_source<F>(
+    fn create_source<F, C>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         name: Option<&str>,
         priority: glib::Priority,
         func: F,
     ) -> glib::Source
     where
         F: FnMut(&Self, glib::IOCondition) -> glib::Continue + 'static,
+        C: IsA<Cancellable>,
     {
-        #[cfg_attr(feature = "cargo-clippy", allow(transmute_ptr_to_ref))]
-        unsafe extern "C" fn trampoline<O: IsA<Socket>>(
-            socket: *mut gio_sys::GSocket,
-            condition: glib_sys::GIOCondition,
-            func: glib_sys::gpointer,
-        ) -> glib_sys::gboolean {
-            let func: &Fragile<
-                RefCell<Box<dyn FnMut(&O, glib::IOCondition) -> glib::Continue + 'static>>,
-            > = transmute(func);
-            let func = func.get();
+        unsafe extern "C" fn trampoline<
+            O: IsA<Socket>,
+            F: FnMut(&O, glib::IOCondition) -> glib::Continue + 'static,
+        >(
+            socket: *mut ffi::GSocket,
+            condition: glib::ffi::GIOCondition,
+            func: glib::ffi::gpointer,
+        ) -> glib::ffi::gboolean {
+            let func: &RefCell<F> = &*(func as *const RefCell<F>);
             let mut func = func.borrow_mut();
             (&mut *func)(
-                &Socket::from_glib_borrow(socket).unsafe_cast(),
+                Socket::from_glib_borrow(socket).unsafe_cast_ref(),
                 from_glib(condition),
             )
-            .to_glib()
+            .into_glib()
         }
-        unsafe extern "C" fn destroy_closure<O>(ptr: glib_sys::gpointer) {
-            Box::<
-                Fragile<RefCell<Box<dyn FnMut(&O, glib::IOCondition) -> glib::Continue + 'static>>>,
-            >::from_raw(ptr as *mut _);
+        unsafe extern "C" fn destroy_closure<O, F>(ptr: glib::ffi::gpointer) {
+            Box::<RefCell<F>>::from_raw(ptr as *mut _);
         }
-        let cancellable = cancellable.to_glib_none();
+        let cancellable = cancellable.map(|c| c.as_ref());
+        let gcancellable = cancellable.to_glib_none();
         unsafe {
-            let source = gio_sys::g_socket_create_source(
+            let source = ffi::g_socket_create_source(
                 self.as_ref().to_glib_none().0,
-                condition.to_glib(),
-                cancellable.0,
+                condition.into_glib(),
+                gcancellable.0,
             );
-            let trampoline = trampoline::<O> as glib_sys::gpointer;
-            glib_sys::g_source_set_callback(
+            let trampoline = trampoline::<O, F> as glib::ffi::gpointer;
+            glib::ffi::g_source_set_callback(
                 source,
-                Some(transmute(trampoline)),
-                into_raw(func),
-                Some(destroy_closure::<O>),
+                Some(transmute::<
+                    _,
+                    unsafe extern "C" fn(glib::ffi::gpointer) -> glib::ffi::gboolean,
+                >(trampoline)),
+                Box::into_raw(Box::new(RefCell::new(func))) as glib::ffi::gpointer,
+                Some(destroy_closure::<O, F>),
             );
-            glib_sys::g_source_set_priority(source, priority.to_glib());
+            glib::ffi::g_source_set_priority(source, priority.into_glib());
 
             if let Some(name) = name {
-                glib_sys::g_source_set_name(source, name.to_glib_none().0);
+                glib::ffi::g_source_set_name(source, name.to_glib_none().0);
             }
 
             from_glib_full(source)
         }
     }
 
-    #[cfg(feature = "futures")]
-    fn create_source_future(
+    fn create_source_future<C: IsA<Cancellable>>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         priority: glib::Priority,
-    ) -> Box<dyn Future<Output = glib::IOCondition> + std::marker::Unpin> {
-        let cancellable: Option<Cancellable> = cancellable.cloned();
+    ) -> Pin<Box<dyn std::future::Future<Output = glib::IOCondition> + 'static>> {
+        let cancellable: Option<Cancellable> = cancellable.map(|c| c.as_ref()).cloned();
 
-        let obj = Fragile::new(self.clone());
-        Box::new(glib::SourceFuture::new(move |send| {
+        let obj = self.clone();
+        Box::pin(glib::SourceFuture::new(move |send| {
             let mut send = Some(send);
-            obj.get().create_source(
+            obj.create_source(
                 condition,
                 cancellable.as_ref(),
                 None,
@@ -406,19 +416,18 @@ impl<O: IsA<Socket>> SocketExtManual for O {
         }))
     }
 
-    #[cfg(feature = "futures")]
-    fn create_source_stream(
+    fn create_source_stream<C: IsA<Cancellable>>(
         &self,
         condition: glib::IOCondition,
-        cancellable: Option<&Cancellable>,
+        cancellable: Option<&C>,
         priority: glib::Priority,
-    ) -> Box<dyn Stream<Item = glib::IOCondition> + std::marker::Unpin> {
-        let cancellable: Option<Cancellable> = cancellable.cloned();
+    ) -> Pin<Box<dyn Stream<Item = glib::IOCondition> + 'static>> {
+        let cancellable: Option<Cancellable> = cancellable.map(|c| c.as_ref()).cloned();
 
-        let obj = Fragile::new(self.clone());
-        Box::new(glib::SourceStream::new(move |send| {
+        let obj = self.clone();
+        Box::pin(glib::SourceStream::new(move |send| {
             let send = Some(send);
-            obj.get().create_source(
+            obj.create_source(
                 condition,
                 cancellable.as_ref(),
                 None,
@@ -435,15 +444,6 @@ impl<O: IsA<Socket>> SocketExtManual for O {
     }
 }
 
-fn into_raw<O, F: FnMut(&O, glib::IOCondition) -> glib::Continue + 'static>(
-    func: F,
-) -> glib_sys::gpointer {
-    let func: Box<
-        Fragile<RefCell<Box<dyn FnMut(&O, glib::IOCondition) -> glib::Continue + 'static>>>,
-    > = Box::new(Fragile::new(RefCell::new(Box::new(func))));
-    Box::into_raw(func) as glib_sys::gpointer
-}
-
 #[cfg(all(not(unix), feature = "dox"))]
 pub trait IntoRawFd {
     fn into_raw_fd(self) -> c_int;
